# rubocop:disable Style/FileName
require 'json'
require 'sassc'

# Pass either a file or a string containing scss and
# recieve a compiled string of css with or without
# style tags.
class SassBuddy
  def initialize(opts)
    @file = opts[:file]
    @string, = opts[:string]
    @container = opts[:container]
    @additions = opts[:additions]
    @require_paths = opts[:require_paths]
    @manifest = opts[:rev_manifest] ? get_manifest(opts[:rev_manifest]) : {}
    @syntax = opts[:sass] ? :sass : :scss
  end
  attr_reader :file, :string, :additions, :container, :require_paths, :manifest

  def compile
    revved(compiled_css) if compiled_css
  end

  def compile_html
    "<style>#{revved(compiled_css)}</style>" if compiled_css
  end

  private

  def get_manifest(file)
    JSON.parse(File.open(file).read)
  end

  def revved(css)
    manifest.each do |k, v|
      css.gsub!(k, v)
    end

    css
  end

  def compiled_css
    SassC::Engine.new(
      stringed_additions + contents,
      syntax: @syntax,
      importer: ImportBuddy,
      filename: file,
      require_paths: require_paths
    ).render
  rescue SassC::SyntaxError
    false
  end

  def stringed_additions
    return '' unless additions

    additions.inject('') do |out, (k, v)|
      out + "$#{k}:##{v};"
    end
  end

  def contents
    @contents ||= begin
      scss = file ? File.open(file).read : string
      scss = "#{container} {#{scss}}" if container
      scss
    end
  end
end

# :nodoc:
class ImportBuddy < SassC::Importer
  def imports(path, _parent_path)
    if path_match(path)
      SassC::Importer::Import.new(path_match(path))
    else
      SassC::Importer::Import.new(path)
    end
  end

  def path_match(path)
    custom_require_path.each do |k, v|
      return File.join(v, path) unless (Regexp.new(k.to_s) =~ path).nil?
    end

    false
  end

  def custom_require_path
    @custom_require_path ||= options[:require_paths]
  end
end
